/**
 * Copyright © 2007-2010 Kirill Gavrilov <kirill@sview.ru>
 *
 * StCore library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * StCore library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.
 * If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __StCoreImpl_h_
#define __StCoreImpl_h_

#include <StCore/StCore.h>
#include <StCore/StDrawer.h>
#include <StCore/StWindowInterface.h>

// Exported class-methods wpappers in StCore library (here!)
ST_EXPORT StWindowInterface* StWindow_new();
ST_EXPORT void StWindow_del(StWindowInterface* );

class ST_LOCAL StCoreImpl : public StRendererInterface {

        private:

    StWindowInterface* stWindow; // real StWindow instaniated here, window created in Render plugin
    StDrawer*          stDrawer; // current Drawer instance
    StString       sViewPath; // main sView libraries path
    StString    stDrawerPath; // current Drawer library path

        public:

    // exports
    StRendererInterface* getLibImpl() { return this; }
    StCoreImpl();
    ~StCoreImpl();
    StWindowInterface* getStWindow() { return stWindow; }
    bool init(const StString& rendererPath, const int& deviceId, const StNativeWin_t* );
    bool open(const StOpenInfo& stOpenInfo);
    void callback(StMessage_t* stMessages) {
        stWindow->callback(stMessages);
        if(stDrawer != NULL) {
            stDrawer->parseCallback(stMessages);
        }
    }
    void stglDraw(unsigned int views) {
        if(stDrawer != NULL) {
            stDrawer->stglDraw(views);
        }
    }

        private:

    // additional
    void closeDrawer();

};

#endif //__StCoreImpl_h_
