/**
 * Copyright © 2010-2011 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#include <StGLWidgets/StGLImageFlatProgram.h>
#include <StGL/StGLResources.h>

StGLImageFlatProgram::StGLImageFlatProgram()
: StGLImageProgram("StGLImageFlatProgram"),
  fGetColorPtr(NULL) {
    fGetColorPtr = &fGetColor;
}

StGLImageFlatProgram::~StGLImageFlatProgram() {
    //
}

void StGLImageFlatProgram::setSmoothFilter(const TextureFilter textureFilter) {
    if(!isValid()) {
        return;
    }
    switch(textureFilter) {
        case FILTER_BLEND: {
            StGLProgram::detachShader(*fGetColorPtr).attachShader(fGetColorBlend).link();
            fGetColorPtr = &fGetColorBlend;
            break;
        }
        case FILTER_NEAREST:
        case FILTER_LINEAR:
        default: {
            StGLProgram::detachShader(*fGetColorPtr).attachShader(fGetColor).link();
            fGetColorPtr = &fGetColor;
            break;
        }
    }
}

bool StGLImageFlatProgram::init() {
    if(!StGLImageProgram::init()) {
        return false;
    }
    const StGLResources aShaders("StGLWidgets");
    StGLVertexShader vShaderMain(StGLProgram::getTitle());
    vShaderMain.initFile(aShaders.getShaderFile("flatImage.shv"));

    StGLFragmentShader fShaderMain(StGLProgram::getTitle());
    fShaderMain.initFile(aShaders.getShaderFile("flatImage.shf"));

    return StGLProgram::create()
        .attachShader(vShaderMain)
        .attachShader(fShaderMain)
        .attachShader(*fGetColorPtr)
        .attachShader(*f2RGBPtr)
        .attachShader(*fCorrectPtr)
        .attachShader(*fGammaPtr)
        .link();
}
