/**
 * Copyright © 2010-2011 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#include <StGLWidgets/StGLImageSphereProgram.h>
#include <StGL/StGLResources.h>

StGLImageSphereProgram::StGLImageSphereProgram()
: StGLImageProgram("StGLImageSphereProgram"),
  fGetColorPtr(NULL),
  fGetColorLinear("StGLImageSphereProgram::fGetColorLinear") {
    fGetColorPtr = &fGetColor;
}

StGLImageSphereProgram::~StGLImageSphereProgram() {
    //
}

void StGLImageSphereProgram::setSmoothFilter(const TextureFilter textureFilter) {
    switch(textureFilter) {
        case FILTER_BLEND:
        case FILTER_LINEAR: {
            if(isValid()) {
                StGLProgram::detachShader(*fGetColorPtr).attachShader(fGetColorLinear).link();
            }
            fGetColorPtr = &fGetColorLinear;
            break;
        }
        case FILTER_NEAREST:
        default: {
            if(isValid()) {
                StGLProgram::detachShader(*fGetColorPtr).attachShader(fGetColor).link();
            }
            fGetColorPtr = &fGetColor;
            break;
        }
    }
}

bool StGLImageSphereProgram::init() {
    if(!StGLImageProgram::init()) {
        return false;
    }
    const StGLResources aShaders("StGLWidgets");
    StGLVertexShader vShaderMain(StGLProgram::getTitle());
    vShaderMain.initFile(aShaders.getShaderFile("sphereImage.shv"));

    StGLFragmentShader fShaderMain(StGLProgram::getTitle());
    fShaderMain.initFile(aShaders.getShaderFile("sphereImage.shf"));

    fGetColorLinear.initFile(aShaders.getShaderFile("sphereGetColorLinear.shf"));

    return StGLProgram::create()
        .attachShader(vShaderMain)
        .attachShader(fShaderMain)
        .attachShader(*fGetColorPtr)
        .attachShader(*f2RGBPtr)
        .attachShader(*fCorrectPtr)
        .attachShader(*fGammaPtr)
        .link();
}
