/**
 * Copyright © 2009-2010 Kirill Gavrilov <kirill@sview.ru>
 *
 * StOutIZ3D library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * StOutIZ3D library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.
 * If not, see <http://www.gnu.org/licenses/>.
 */

#include "StOutIZ3DShaders.h"
#include <StThreads/StProcess.h>

void StOutIZ3DShaders::setMode(const int& mode) {
    this->mode = mode;
    switch(mode) {
        case IZ3D_TABLE_OLD:
        case IZ3D_TABLE_NEW: {
            shBack = &shBackTable;
            shFront = &shFrontTable;
            break;
        }
        default: {
            shBack = &shBackClassic;
            shFront = &shFrontClassic;
        }
    }
}

bool StOutIZ3DShaders::init() {
    // shaders data
    const StString VSHADER                    = "vIZ3D.shv";
    const StString FSHADER_IZ3D_BACK_CLASSIC  = "fIZ3DBackClassic.shf";
    const StString FSHADER_IZ3D_FRONT_CLASSIC = "fIZ3DFrontClassic.shf";
    const StString FSHADER_IZ3D_BACK_TABLE    = "fIZ3DBackTable.shf";
    const StString FSHADER_IZ3D_FRONT_TABLE   = "fIZ3DFrontTable.shf";

    StString aShadersError("StOutIZ3D Plugin, Failed to init Shaders");
    const StString aShadersRoot = StProcess::getStCoreFolder() + "shaders" + SYS_FS_SPLITTER
                                   + "StOutIZ3D" + SYS_FS_SPLITTER;

    // initialize shaders
    StGLVertexShader stVertexShader("iZ3D"); // common vertex shader
    if(!stVertexShader.initFile(aShadersRoot + VSHADER)) {
        stError(aShadersError);
        return false;
    }

    StGLFragmentShader stFBackClassic(shBackClassic.getTitle());
    if(!stFBackClassic.initFile(aShadersRoot + FSHADER_IZ3D_BACK_CLASSIC)) {
        stError(aShadersError);
        return false;
    }
    shBackClassic.create().attachShader(stVertexShader).attachShader(stFBackClassic).link();

    StGLFragmentShader stFFrontClassic(shFrontClassic.getTitle());
    if(!stFFrontClassic.initFile(aShadersRoot + FSHADER_IZ3D_FRONT_CLASSIC)) {
        stError(aShadersError);
        return false;
    }
    shFrontClassic.create().attachShader(stVertexShader).attachShader(stFFrontClassic).link();

    StGLFragmentShader stFBackTable(shBackTable.getTitle());
    if(!stFBackTable.initFile(aShadersRoot + FSHADER_IZ3D_BACK_TABLE)) {
        stError(aShadersError);
        return false;
    }
    shBackTable.create().attachShader(stVertexShader).attachShader(stFBackTable).link();

    StGLFragmentShader stFFrontTable(shFrontTable.getTitle());
    if(!stFFrontTable.initFile(aShadersRoot + FSHADER_IZ3D_FRONT_TABLE)) {
        stError(aShadersError);
        return false;
    }
    shFrontTable.create().attachShader(stVertexShader).attachShader(stFFrontTable).link();

    // get shaders' variables' locations
    StGLVarLocation uniTTLocBack  = shBackTable.getUniformLocation("textT");
    StGLVarLocation uniTTLocFront = shFrontTable.getUniformLocation("textT");

    if(shBackTable.isValid() && uniTTLocBack.isValid()) {
        shBackTable.use();
        glUniform1i(uniTTLocBack, 2); // GL_TEXTURE2
        shBackTable.unuse();
    }

    if(shFrontTable.isValid() && uniTTLocFront.isValid()) {
        shFrontTable.use();
        glUniform1i(uniTTLocFront, 2); // GL_TEXTURE2
        shFrontTable.unuse();
    }

    return (shBackClassic.isValid() && shFrontClassic.isValid())
        || (  shBackTable.isValid() && shFrontTable.isValid() && uniTTLocBack.isValid() && uniTTLocFront.isValid());
}

void StOutIZ3DShaders::release() {
    shBackClassic.release();
    shFrontClassic.release();
    shBackTable.release();
    shFrontTable.release();
}
