/**
 * Copyright © 2009-2010 Kirill Gavrilov <kirill@sview.ru>
 *
 * StOutIZ3D library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * StOutIZ3D library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.
 * If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __StOutIZ3DShaders_h_
#define __StOutIZ3DShaders_h_

#include <StGLStereo/StGLStereoFrameBuffer.h>

/**
 * Helper class to access IZ3D shaders array.
 */
class ST_LOCAL StOutIZ3DShaders {

        private:

    StGLStereoFrameBuffer::StGLStereoProgram* shBack;  // program for master window
    StGLStereoFrameBuffer::StGLStereoProgram* shFront; // program for slave window

    StGLStereoFrameBuffer::StGLStereoProgram  shBackClassic; // classic shaders - computations
    StGLStereoFrameBuffer::StGLStereoProgram shFrontClassic;
    StGLStereoFrameBuffer::StGLStereoProgram    shBackTable; // new shaders - values getted from optimized tables
    StGLStereoFrameBuffer::StGLStereoProgram   shFrontTable;

    int mode; // switch old classes table / new glasses table / classic computations

        public:

    enum {
        IZ3D_TABLE_OLD = 0,
        IZ3D_TABLE_NEW = 1,
        IZ3D_CLASSIC = 2,
    };

    StOutIZ3DShaders()
    : shBack(NULL),
      shFront(NULL),
      shBackClassic ("iZ3D Classic Back"),
      shFrontClassic("iZ3D Classic Front"),
      shBackTable ("iZ3D Table Back"),
      shFrontTable("iZ3D Table Front"),
      mode(IZ3D_TABLE_OLD) {
        //
        shBack = &shBackClassic;
        shFront = &shFrontClassic;
    }

    ~StOutIZ3DShaders() {
        //
    }

    const int& getMode() const {
        return mode;
    }

    void setMode(const int& mode);
    bool init();
    void release();

    StGLStereoFrameBuffer::StGLStereoProgram* master() {
        return shBack;
    }

    StGLStereoFrameBuffer::StGLStereoProgram* slave() {
        return shFront;
    }

};

#endif //__StOutIZ3DShaders_h_
