/**
 * Copyright © 2009-2011 Kirill Gavrilov <kirill@sview.ru>
 *
 * StOutPageFlip library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * StOutPageFlip library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.
 * If not, see <http://www.gnu.org/licenses/>.
 */

#if(defined(_WIN32) || defined(__WIN32__))
#ifndef __StDXNVWindow_h_
#define __StDXNVWindow_h_

#include "StDXManager.h"
#include "StDXNVSurface.h"

#include <StCore/StCore.h>       // header for Stereo Output Core
#include <StThreads/StThreads.h>

class StDXNVWindow {

        private:

    unsigned char*    myBufferL;
    unsigned char*    myBufferR;
    size_t           myFboSizeX;
    size_t           myFboSizeY;

    HWND               myWinD3d;
    HWND            myWinMaster;
    StStringUtfWide  myWinClass;

    StHandle<StDXManager>   myDxManager;
    StHandle<StDXNVSurface> myDxSurface;

    StMonitor         myMonitor;
    StWindowInterface*  myStWin;

    StMutex             myMutex;

    HANDLE          hEventReady;
    HANDLE           hEventQuit; // quit message thread event
    HANDLE           hEventShow;
    HANDLE           hEventHide;
    HANDLE         hEventUpdate;
    bool             myIsOwnWin;

    bool myMouseState[ST_MOUSE_MAX_ID + 1];
    bool myVKeyState[255];

    bool initWinAPIWindow();

        public:

    StDXNVWindow(const size_t       theFboSizeX,
                 const size_t       theFboSizeY,
                 const StMonitor&   theMonitor,
                 StWindowInterface* theStWin);

    ~StDXNVWindow();

    size_t getD3dSizeX() const {
        return myMonitor.getVRect().width();
    }

    size_t getD3dSizeY() const {
        return myMonitor.getVRect().height();
    }

    LRESULT wndProcFunction(HWND , UINT , WPARAM , LPARAM );

    bool hasOwnWindow() const {
        return myIsOwnWin;
    }

    StWindowInterface* getStWindow() {
        return myStWin;
    }

    void updateMouseBtn(const int btnId, bool newState);
    void updateKeyState(const int vkeyId, bool newState);

    void waitReady() {
        WaitForSingleObject(hEventReady, INFINITE);
    }

    void show() {
        ResetEvent(hEventHide);
        SetEvent(hEventShow);
    }

    void hide() {
        ResetEvent(hEventShow);
        SetEvent(hEventHide);
    }

    void update() {
        SetEvent(hEventUpdate);
    }

    bool isInUpdate() {
        return (WaitForSingleObject(hEventUpdate, (DWORD )0) != WAIT_TIMEOUT);
    }

    void lockLRBuffers() {
        myMutex.lock();
    }

    void unlockLRBuffers() {
        myMutex.unlock();
    }

    unsigned char* getBuffLeft() {
        return myBufferL;
    }

    unsigned char* getBuffRight() {
        return myBufferR;
    }

    void dxLoop();

    void quit() {
        SetEvent(hEventQuit);
    }

};

#endif //__StDXNVWindow_h_
#endif //_WIN32
