/**
 * Copyright © 2010 Kirill Gavrilov <kirill@sview.ru>
 *
 * StOutPageFlip library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * StOutPageFlip library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.
 * If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __StGLControlED_h_
#define __StGLControlED_h_

#include "StGLDeviceControl.h"

#include <StThreads/StTimer.h>

class ST_LOCAL StGLControlED : public StGLDeviceControl {

        private:

    static const double DELAY_MS; // 500

    class StEDProgram;
    StEDProgram* stProgramOn;
    StEDProgram* stProgramOff;
    StEDProgram* stProgramBlack;

    StTimer stTimerCode;  // delays timers
    StTimer stTimerBlack; //

        public:

    virtual int getSizeY() const {
        return 10;
    }

    virtual int getSlaveId() const {
        return SLAVE_HLINE_TOP;
    }

    StGLControlED();
    virtual ~StGLControlED();

    virtual bool isActive() const {
        return stTimerCode.isOn() || stTimerBlack.isOn();
    }

    virtual void setMode(int mode) {
        if(stTimerCode.isOn() && stTimerCode.getElapsedTimeInMilliSec() > DELAY_MS) {
            stTimerCode.stop();
            stTimerBlack.restart();
        } else if(stTimerBlack.isOn() && stTimerBlack.getElapsedTimeInMilliSec() > DELAY_MS) {
            stTimerBlack.stop();
        }
        if(!stTimerCode.isOn() && !stTimerBlack.isOn() && (mode != StGLDeviceControl::getMode())) {
            StGLDeviceControl::setMode(mode);
            stTimerCode.restart();
        }
    }

    virtual double quitMS() {
        return 4.0 * DELAY_MS;
    }

    virtual bool stglInit();
    virtual void stglRelease();
    virtual void stglDraw(unsigned int view, const int windowWidth, const int windowHeight);

};

#endif //__StGLControlED_h_
