/**
 * Copyright © 2009-2011 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#include <StCore/StRendererInfo.h>
#include <StCore/StRendererPlugin.h>
#include <StFile/StFileNode.h>

StRendererInfo::StRendererInfo()
: rendererPath(),
  about(),
  deviceList(),
  bValid(false) {
    //
}

StRendererInfo::StRendererInfo(const StRendererInfo& toCopy)
: rendererPath(toCopy.rendererPath),
  about(toCopy.about),
  deviceList(toCopy.deviceList),
  bValid(toCopy.bValid) {
    //
}

StRendererInfo::StRendererInfo(const StString& rendererPath)
: rendererPath(rendererPath),
  about(),
  deviceList(),
  bValid(false) {
    StRendererPlugin stRenderer;
    if(stRenderer.InitLibrary(rendererPath)) {
        const StStereoDeviceInfo_t* stDeviceInfo = stRenderer.GetDevicesList();
        if(stDeviceInfo != NULL) {
            deviceList = StStereoDeviceInfoList(stDeviceInfo);
            bValid = true;
        }
    }
}

const StRendererInfo& StRendererInfo::operator=(const StRendererInfo& toCopy) {
    if(this != &toCopy) {
        rendererPath = toCopy.rendererPath;
        about        = toCopy.about;
        deviceList   = toCopy.deviceList;
    }
    return (*this);
}

StString StRendererInfo::getTitle() const {
    StString aDummy, aFile, aTitle;
    StFileNode::getFolderAndFile(rendererPath, aDummy, aFile);
    StFileNode::getNameAndExtension(aFile, aTitle, aDummy);
    return aTitle;
}

int StRendererInfo::getSupportLevel() const {
    int maxSupportLevel = ST_DEVICE_SUPPORT_NONE;
    for(size_t devId = 0; devId < deviceList.size(); devId++) {
        int supportLevel = deviceList[devId].getDetectionLevel();
        maxSupportLevel = supportLevel > maxSupportLevel ? supportLevel : maxSupportLevel;
    }
    return maxSupportLevel;
}

bool StRendererInfo::operator==(const StRendererInfo& theCompare) const {
    if(&theCompare == this) {
        return true;
    }
#if(defined(_WIN32) || defined(__WIN32__))
    return theCompare.rendererPath.isEqualsIgnoreCase(rendererPath);
#else
    return theCompare.rendererPath == rendererPath;
#endif
}

bool StRendererInfo::operator!=(const StRendererInfo& theCompare) const {
    if(&theCompare == this) {
        return false;
    }
#if(defined(_WIN32) || defined(__WIN32__))
    return !theCompare.rendererPath.isEqualsIgnoreCase(rendererPath);
#else
    return theCompare.rendererPath != rendererPath;
#endif
}

StString StRendererInfo::toString() const {
    return StString("Renderer Path = '") + rendererPath + "'. Full Device list:\n" + deviceList.toString();
}
