/**
 * Copyright © 2009-2010 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#include <StSettings/StSettings.h>
#include <StThreads/StProcess.h>

namespace {
    static StLibrary ST_CONFIG_LIB;
    static StMutex ST_CONFIG_MUTEX;
    static size_t ST_CONFIG_COUNTER = 0;
    static const char LIB_NAME[] = "StSettings";
};

StLibrary& StSettings::GetLibrary() {
    return ST_CONFIG_LIB;
}

StMutex& StSettings::GetMutex() {
    return ST_CONFIG_MUTEX;
}

size_t& StSettings::GetUseCounter() {
    return ST_CONFIG_COUNTER;
}

StSettings::StSettings(const StString& settingsSet)
: StConfig(settingsSet) {
    //
}

StSettings::~StSettings() {
    //
}

// INIT library function - called by host application to use library classes
int StSettings::INIT() {
    GetMutex().lock();
    if(GetUseCounter() > 0) {
        ++GetUseCounter();
        GetMutex().unlock();
        return STERROR_LIBNOERROR;
    }

    if(!GetLibrary().load(StProcess::getStCoreFolder() + LIB_NAME)) {
        GetMutex().unlock();
        return STERROR_LIBLOADFAILED;
    }

    // get functions addresses in library
    GetFunctions().load(GetLibrary());
    if(GetFunctions().isNull()) {
        GetMutex().unlock();
        FREE();
        return STERROR_LIBFUNCTIONNOTFOUND;
    }
    // initialization success
    ++GetUseCounter();
    GetMutex().unlock();
    return STERROR_LIBNOERROR;
}

void StSettings::FREE() {
    GetMutex().lock();
    if(GetUseCounter() > 0) {
        --GetUseCounter();
    }
    if(GetUseCounter() > 0) {
        // library still in use
        GetMutex().unlock();
        return;
    }

    GetLibrary().close();
    GetFunctions().nullify();
    GetMutex().unlock();
}
