/**
 * Copyright © 2009-2011 Kirill Gavrilov <kirill@sview.ru>
 *
 * StCore library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * StCore library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.
 * If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __StRendererInfo_h_
#define __StRendererInfo_h_

#include "StStereoDeviceInfoList.h"

class ST_LOCAL StRendererInfo {

        private:

    StString rendererPath;
    StString about;
    StStereoDeviceInfoList deviceList;
    bool bValid;

        public:

    enum {
        DEVICE_AUTO = -1,
    };

    StRendererInfo();
    StRendererInfo(const StRendererInfo& toCopy);
    StRendererInfo(const StString& rendererPath);

    ~StRendererInfo() {
        //
    }

    const StRendererInfo& operator=(const StRendererInfo& toCopy);

    StString getTitle() const;

    const StString& getPath() const {
        return rendererPath;
    }

    const StStereoDeviceInfoList& getDeviceList() const {
        return deviceList;
    }

    bool isValid() const {
        return bValid;
    }

    int getSupportLevel() const;

    bool operator==(const StRendererInfo& compare) const;

    bool operator!=(const StRendererInfo& compare) const;

    bool operator>(const StRendererInfo& compare) const {
        if(&compare == this) {
            return false;
        }
        return this->getSupportLevel() > compare.getSupportLevel();
    }

    bool operator<(const StRendererInfo& compare) const {
        if(&compare == this) {
            return false;
        }
        return this->getSupportLevel() < compare.getSupportLevel();
    }

    bool operator>=(const StRendererInfo& compare) const {
        if(&compare == this) {
            return true;
        }
        return this->getSupportLevel() >= compare.getSupportLevel();
    }

    bool operator<=(const StRendererInfo& compare) const {
        if(&compare == this) {
            return true;
        }
        return this->getSupportLevel() <= compare.getSupportLevel();
    }

    StString toString() const;

};

#endif //__StRendererInfo_h_
