/**
 * Copyright © 2009 Kirill Gavrilov <kirill@sview.ru>
 *
 * StCore library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * StCore library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.
 * If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __StStereoDeviceInfo_h_
#define __StStereoDeviceInfo_h_

#include "StStereoDeviceInfo_t.h"

#ifdef __cplusplus

#include <StStrings/StString.h>

class ST_LOCAL StStereoDeviceInfo {

        private:

    StString    stringId;
    StString        name;
    StString description;
    int   detectionLevel;

        public:

    StStereoDeviceInfo() : stringId(), name(), description(), detectionLevel(ST_DEVICE_SUPPORT_NONE) {
        //
    }

    StStereoDeviceInfo(const StString& stringId, const StString& name, const StString& description, const int& detectionLevel)
        : stringId(stringId), name(name), description(description), detectionLevel(detectionLevel) {
        //
    }

    StStereoDeviceInfo(const StStereoDeviceInfo& toCopy)
        : stringId(stringId), name(toCopy.name), description(toCopy.description), detectionLevel(toCopy.detectionLevel) {
        //
    }

    StStereoDeviceInfo(const StStereoDeviceInfo_t* toCopy)
    : stringId(StString(toCopy->stringId)),
      name(StString(toCopy->name)),
      description(StString(toCopy->description)),
      detectionLevel(toCopy->detectionLevel) {
        //
    }

    ~StStereoDeviceInfo() {
        //
    }

    const StStereoDeviceInfo& operator=(const StStereoDeviceInfo& toCopy) {
        if(this != &toCopy) {
            stringId = toCopy.stringId;
            name = toCopy.name;
            description = toCopy.description;
            detectionLevel = toCopy.detectionLevel;
        }
        return (*this);
    }

    const StString& getStringId() const {
        return stringId;
    }

    void setStringId(const StString& stringId) {
        this->stringId = stringId;
    }

    const StString& getName() const {
        return name;
    }

    void setName(const StString& name) {
        this->name = name;
    }

    const StString& getDescription() const {
        return description;
    }

    void setDescription(const StString& description) {
        this->description = description;
    }

    const int& getDetectionLevel() const {
        return detectionLevel;
    }

    void setDetectionLevel(const int& detectionLevel) {
        this->detectionLevel = detectionLevel;
    }

    bool operator==(const StStereoDeviceInfo& compare) const {
        if(&compare == this) {
            return true;
        }
        return compare.stringId == this->stringId;
    }

    bool operator!=(const StStereoDeviceInfo& compare) const {
        if(&compare == this) {
            return false;
        }
        return compare.stringId != this->stringId;
    }

    bool operator>(const StStereoDeviceInfo& compare) const {
        if(&compare == this) {
            return false;
        }
        return this->detectionLevel > compare.detectionLevel;
    }

    bool operator<(const StStereoDeviceInfo& compare) const {
        if(&compare == this) {
            return false;
        }
        return this->detectionLevel < compare.detectionLevel;
    }

    bool operator>=(const StStereoDeviceInfo& compare) const {
        if(&compare == this) {
            return true;
        }
        return this->detectionLevel >= compare.detectionLevel;
    }

    bool operator<=(const StStereoDeviceInfo& compare) const {
        if(&compare == this) {
            return true;
        }
        return this->detectionLevel <= compare.detectionLevel;
    }

    /**
     * Warning, struct contains only pointers to class strings,
     * so - class should not be deallocated while struct in use!
     *
     * Method created for special use (to create global STATIC plugin variable, available for StCore),
     * do not use it in other ways to prevent memory corruption.
     */
    StStereoDeviceInfo_t getStruct() const {
        StStereoDeviceInfo_t infoStruct;
        stMemSet(&infoStruct, 0, sizeof(StStereoDeviceInfo_t));
        infoStruct.stringId = (stUtf8_t* )stringId.toCString();
        infoStruct.name = (stUtf8_t* )name.toCString();
        infoStruct.description = (stUtf8_t* )description.toCString();
        infoStruct.detectionLevel = detectionLevel;
        return infoStruct;
    }

    StString toString() const {
        return StString(detectionLevel) + ':' + stringId + ':' + name + ':' + description;
    }

};

#endif //__cplusplus
#endif //__StStereoDeviceInfo_h_
