/**
 * OpenGL helper functions and wrappers.
 * Copyright © 2009-2011 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StOpenGL_h_
#define __StOpenGL_h_

// GLEW headers
#include <gl/glew.h>
#if (defined(_WIN32) || defined(__WIN32__))
    #include <gl/wglew.h>  // wglew - Windows GL extensions
#elif (defined(__APPLE__))
    #include <OpenGL/OpenGL.h>
#elif (defined(__linux__) || defined(__linux))
    #include <gl/glxew.h>  // glxew - GLX extensions
#endif

#include <StTemplates/StRect.h>

/**
 * Just return OpenGL device information for logging.
 */
StString stglInfo();

inline void stglResizeViewport(GLsizei sizeX, GLsizei sizeY) {
    if(sizeY == 0) {  // prevent a Divide by Zero by
        sizeY = 1;    // making height equal one
    }
    glViewport(0, 0, sizeX, sizeY);
}

inline void stglResize(const StRect<GLint>& winRect) {
    stglResizeViewport(winRect.width(), winRect.height());
}

/**
 * Enable VSync (vertical sync).
 */
inline bool stglSetVSync(bool theVSyncOn = true) {
    // TODO (Kirill Gavrilov#5#) check flags on drivers' control
    GLint aSyncInt = (theVSyncOn ? 1 : 0);
#if (defined(_WIN32) || defined(__WIN32__))
    if(wglSwapIntervalEXT) {
        wglSwapIntervalEXT(aSyncInt);
        return true;
    }
#elif (defined(__APPLE__))
    return CGLSetParameter(CGLGetCurrentContext(), kCGLCPSwapInterval, &aSyncInt) == kCGLNoError;
#else
    if(glXSwapIntervalSGI != NULL) {
        glXSwapIntervalSGI(aSyncInt);
        return true;
    }
#endif
    return false;
}

#endif //__StOpenGL_h_
