/**
 * Copyright © 2011 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StGLMemInfo_h_
#define __StGLMemInfo_h_

#include <gl/glew.h>
#include <stTypes.h>

class ST_LOCAL StGLMemInfo {

        private:

    // dedicated (on-board) GPU memory
    GLint myDedicated;
    GLint myDedicatedFree;
    GLint myLargestFreeBlock;
    // total video memory (GPU + RAM part dedicated for video)
    GLint myTotal;
    GLint myTotalFree;

    // use GL_ATI_meminfo extension
    // http://www.opengl.org/registry/specs/ATI/meminfo.txt
    bool readForATI() {
        //#define GL_VBO_FREE_MEMORY_ATI 0x87FB
        //#define GL_TEXTURE_FREE_MEMORY_ATI 0x87FC
        //#define GL_RENDERBUFFER_FREE_MEMORY_ATI 0x87FD
        GLint aMemInfo[4]; stMemSet(aMemInfo, -1, sizeof(aMemInfo));
        glGetIntegerv(0x87FB, aMemInfo);
        // total memory free in the pool
        myDedicatedFree    = aMemInfo[0];
        // largest available free block in the pool
        myLargestFreeBlock = aMemInfo[1];
        // total auxiliary memory free
        myTotalFree = aMemInfo[2];
        // largest auxiliary free block aMemInfo[3];
        return aMemInfo[0] >= 0;
    }

    // use GL_NVX_gpu_memory_info extension
    // http://developer.download.nvidia.com/opengl/specs/GL_NVX_gpu_memory_info.txt
    bool readForNVIDIA() {
        //#define GL_GPU_MEMORY_INFO_DEDICATED_VIDMEM_NVX 0x9047
        //#define GL_GPU_MEMORY_INFO_TOTAL_AVAILABLE_MEMORY_NVX 0x9048
        //#define GL_GPU_MEMORY_INFO_CURRENT_AVAILABLE_VIDMEM_NVX 0x9049
        //#define GL_GPU_MEMORY_INFO_EVICTION_COUNT_NVX 0x904A
        //#define GL_GPU_MEMORY_INFO_EVICTED_MEMORY_NVX 0x904B
        // dedicated video memory, total size (in kb) of the GPU memory
        glGetIntegerv(0x9047, &myDedicated);
        // total available memory, total size (in Kb) of the memory available for allocations
        glGetIntegerv(0x9048, &myTotal); ///???
        //current available dedicated video memory (in kb), currently unused GPU memory
        glGetIntegerv(0x9049, &myDedicatedFree);
        return myDedicated >= 0;
    }

    static StString getMiB(GLint theValue) {
        return (theValue >= 0) ? (StString(theValue / 1024) + " MiB") : StString("N/A");
    }

        public:

    StGLMemInfo()
    : myDedicated(-1),
      myDedicatedFree(-1),
      myLargestFreeBlock(-1),
      myTotal(-1),
      myTotalFree(-1) {
        readForATI();
        readForNVIDIA();
    }

    StString toString() const {
        return StString("  GPU memory:         ") + getMiB(myDedicated) + '\n'
             + StString("  GPU free memory:    ") + getMiB(myDedicatedFree) + '\n'
             + StString("  Largest free block: ") + getMiB(myLargestFreeBlock) + '\n'
             + StString("  Total memory:       ") + getMiB(myTotal) + '\n'
             + StString("  Free memory:        ") + getMiB(myTotalFree) + '\n';
    }

};

#endif //__StGLMemInfo_h_
