/**
 * Copyright © 2010 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StGLVarLocation_h_
#define __StGLVarLocation_h_

#include <stTypes.h>

/**
 * Simple class represents GLSL program variable location.
 */
class ST_LOCAL StGLVarLocation {

        public:

    static const GLint NO_LOC = -1;

        private:

    GLint location;

        public:

    /**
     * Construct a special location which actually means NO location.
     */
    StGLVarLocation() : location(NO_LOC) {}
    StGLVarLocation(const GLint location) : location(location) {}
    virtual ~StGLVarLocation() {}

    /**
     * Note you may safely put invalid location in functions like glUniform*
     * - the data passed in will be silently ignored.
     * @return true if location is not equal to -1.
     */
    bool isValid() const { return location != NO_LOC; }

    /**
     * Convert operators help silently put object to GL functions like glUniform*.
     */
    operator const GLint() const { return location; }
    operator GLint() { return location; }

};

#endif //__StGLVarLocation_h_
