/**
 * Copyright © 2010 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StGLEW_h__
#define __StGLEW_h__

// GLEW headers
#include <gl/glew.h>
#if(defined(_WIN32) || defined(__WIN32__))
    #include <gl/wglew.h>  // wglew - Windows GL extensions
#elif(defined(__linux__) || defined(__linux))
    #include <gl/glxew.h>  // glxew - GLX extensions
#endif

#include <StStrings/StLogger.h>
#include <StThreads/StMutex.h>

/// TODO (Kirill Gavrilov#8) migrate to GLEW MT to be safe from multithreading/multiple GL contexts errors

/**
 * This is simple wrapper to perform GLEW initialization only once
 */
class ST_LOCAL StGLEW {

        public:

    StMutex stMutex;
    bool bInitialized;

    StGLEW()
    : stMutex(),
      bInitialized(false) {
        //
    }

    ~StGLEW() {
        //
    }

    static bool init() {
        getInstance().stMutex.lock();
            // very important! needed for non aligned data transfer!
            // set here globally to avoid hidden errors
            glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
            glPixelStorei(GL_PACK_ALIGNMENT,   1);
            if(getInstance().bInitialized) {
                getInstance().stMutex.unlock();
                return true;
            }
            // INIT GLEW library
            GLenum err = glewInit();
            if(err != GLEW_OK) {
                getInstance().stMutex.unlock();
                return false;
            }
            ST_DEBUG_LOG("StGLEW INIT");
        getInstance().stMutex.unlock();
        return true;
    }

    static StGLEW& getInstance() {
        static StGLEW stGLEW = StGLEW();
        return stGLEW;
    }

};

#endif //__StGLEW_h__
