/**
 * Copyright © 2010-2012 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StGLImageProgram_h_
#define __StGLImageProgram_h_

#include <StGL/StGLSaturationMatrix.h>
#include <StGL/StGLBrightnessMatrix.h>
#include <StGLMesh/StGLMesh.h>
#include <StImage/StImage.h>
#include <StSettings/StFloat32Param.h>

class ST_LOCAL StGLImageProgram : public StGLMeshProgram {

        public:

    typedef enum tagTextureFilter {
        FILTER_NEAREST, //!< ugly filter
        FILTER_LINEAR,  //!< linear filter
        FILTER_BLEND,   //!< blend deinterlace filter
    } TextureFilter;

        protected:

    StGLVarLocation uniTexMainDataLoc;
    StGLVarLocation uniTexUVDataLoc;
    StGLVarLocation uniTexSizePxLoc;
    StGLVarLocation uniTexelSizePxLoc;
    StGLVarLocation uniColorProcessingLoc;
    StGLVarLocation uniGammaLoc;

    StGLFragmentShader  fGetColor;
    StGLFragmentShader  fGetColorBlend;

    // color conversion
    StGLFragmentShader* f2RGBPtr;
    StGLFragmentShader  fRGB2RGB;
    StGLFragmentShader  fRGBA2RGB;
    StGLFragmentShader  fGray2RGB;
    StGLFragmentShader  fYUV2RGB;
    StGLFragmentShader  fYUV2RGBjpeg;
    // color correction
    StGLFragmentShader* fCorrectPtr;
    StGLFragmentShader  fCorrectNO;
    StGLFragmentShader  fCorrectON;
    StGLBrightnessMatrix myBrightness;
    StGLSaturationMatrix mySaturation;
    StGLVec3             myColorScale; //!< scale filter for de-anaglyph processing
    // gamma correction
    StGLFragmentShader* fGammaPtr;
    StGLFragmentShader  fGammaNO;
    StGLFragmentShader  fGammaON;

        protected:

    void setCorrectionUniform();
    void setGammaUniform();

    void setupCorrectionShader(const float );

    bool isNoColorScale() const;

        public:

    StGLImageProgram(const StString& theTitle);

    virtual ~StGLImageProgram();

    /**
     * Setup color model conversion
     */
    void setupSrcColorShader(StImage::ImgColorModel theColorModel);

    void setTextureSizePx(const StGLVec2& stVec2) {
        glUniform2fv(uniTexSizePxLoc, 1, stVec2);
        glUniform2f(uniTexelSizePxLoc, 1.0f / stVec2.x(), 1.0f / stVec2.y());
    }

    void setTextureMainDataSize(const StGLVec4& texDataVec4) {
        glUniform4fv(uniTexMainDataLoc, 1, texDataVec4);
    }

    void setTextureUVDataSize(const StGLVec4& texDataVec4) {
        glUniform4fv(uniTexUVDataLoc, 1, texDataVec4);
    }

    void setColorScale(const StGLVec3& theScale);
    void resetColorScale();

    /**
     * Initialize default shaders, nothing more.
     */
    virtual bool init();

    /**
     * Perform (re)link and search variables locations.
     */
    virtual bool link();

        public: //!< Properties

    struct {

        StHandle<StFloat32Param> gamma;      //!< gamma correction coefficient
        StHandle<StFloat32Param> brightness; //!< brightness level
        StHandle<StFloat32Param> saturation; //!< saturation value

    } params;

        private: //!< callback Slots

    void doGammaChanged(const float );

};

#endif //__StGLImageProgram_h_
