/**
 * Copyright © 2009-2010 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StGLMenuProgram_h_
#define __StGLMenuProgram_h_

#include <StGL/StGLProgram.h>

class ST_LOCAL StGLMenuProgram : public StGLProgram {

        private:

    StGLVarLocation uniProjMatLoc;
    StGLVarLocation uniColorLoc;
    StGLVarLocation atrVVertexLoc;

        public:

    StGLMenuProgram()
    : StGLProgram("StGLMenuProgram"),
      uniProjMatLoc(),
      uniColorLoc(),
      atrVVertexLoc() {
        //
    }

    virtual ~StGLMenuProgram() {
        //
    }

    StGLVarLocation getVVertexLoc() const {
        return atrVVertexLoc;
    }

    void setProjMat(const StGLMatrix& stProjMat) {
        glUniformMatrix4fv(uniProjMatLoc, 1, GL_FALSE, stProjMat);
    }

    void setColor(const StGLVec4& color, const GLfloat opacityValue) {
        glUniform4fv(uniColorLoc, 1, StGLVec4(color.rgb(), color.a() * opacityValue));
    }

    virtual bool init() {
        const char VERTEX_SHADER[] =
           "uniform mat4 uProjMat; \
            attribute vec4 vVertex; \
            void main(void) { \
                gl_Position = uProjMat * vVertex; \
            }";

        const char FRAGMENT_SHADER[] =
           "uniform vec4 uColor; \
            void main(void) { \
                gl_FragColor = uColor; \
            }";

        StGLVertexShader stVertexShader(StGLProgram::getTitle());
        stVertexShader.init(VERTEX_SHADER);

        StGLFragmentShader stFragmentShader(StGLProgram::getTitle());
        stFragmentShader.init(FRAGMENT_SHADER);
        if(!StGLProgram::create().attachShader(stVertexShader).attachShader(stFragmentShader).link()) {
            return false;
        }

        uniProjMatLoc = StGLProgram::getUniformLocation("uProjMat");
        uniColorLoc   = StGLProgram::getUniformLocation("uColor");
        atrVVertexLoc = StGLProgram::getAttribLocation("vVertex");

        return uniProjMatLoc.isValid()
            && uniColorLoc.isValid()
            && atrVVertexLoc.isValid();
    }

};

#endif //__StGLMenuProgram_h_
