/**
 * Copyright © 2009-2011 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StGLRootWidget_h_
#define __StGLRootWidget_h_

#include <StGLWidgets/StGLShare.h>
#include <StGLWidgets/StGLWidget.h>

/**
 * Full OpenGL-window widget, must be ROOT for other widgets.
 */
class ST_LOCAL StGLRootWidget : public StGLWidget {

        private:

    /// TODO (Kirill Gavrilov#9) - replace with list to StHandle from some base class
    StGLSharePointer** myShareArray; //!< resources shared within GL context (commonly used)
    size_t              myShareSize;
    StGLProjCamera     myProjCamera; //!< projection camera
    StGLMatrix         myScrProjMat; //!< projection matrix within translation to the screen

    StRectD_t              myRectGl; //!< rectangle in GL coordinates
    GLdouble             myScaleGlX; //!< scale factor to optimize convertion from Pixels -> GL coordinates
    GLdouble             myScaleGlY; //!< scale factor to optimize convertion from Pixels -> GL coordinates
    StPointD_t     cursorZo; // mouse cursor position

        public:

    /**
     * Main constructor.
     */
    StGLRootWidget();

    /**
     * Destructor.
     */
    virtual ~StGLRootWidget();

    virtual const StString& getClassName();

    GLdouble getRootScaleX() const {
        return myScaleGlX;
    }

    GLdouble getRootScaleY() const {
        return myScaleGlY;
    }

    StRectD_t getRootRectGl() const {
        return myRectGl;
    }

    /**
     * Convert pixel coordinates (absolute) into GL coordinates.
     */
    StRectD_t getRectGl(const StRectI_t& theRectPx) const;

    /**
     * Convert pixel coordinates (absolute) into GL coordinates.
     * Array should be 4 items length or you will got an exception!
     */
    void getRectGl(const StRectI_t& theRectPx,
                   StArray<StGLVec2>& theVertices,
                   const size_t theFromId = 0) const;

    /**
     * Access the shared resource by unique id.
     */
    StGLSharePointer* getShare(const size_t theResId);

    /**
     * Generate next unique id for shared resource.
     */
    static size_t generateShareId();

    StGLProjCamera* getCamera() {
        return &myProjCamera;
    }

    /**
     * Returns camera projection matrix within to-screen displacement
     * thus it can be used for vertices given in only 2D-coordinates.
     */
    const StGLMatrix& getScreenProjection() const {
        return myScrProjMat;
    }

    /**
     * Returns mouse cursor position in GL coordinates.
     */
    StPointD_t getCursorZo() {
        return cursorZo;
    }

    virtual void stglUpdate(const StPointD_t& cursorZo);
    virtual void stglResize(const StRectI_t& winRectPx);

};

#endif //__StGLRootWidget_h_
