/**
 * Copyright © 2009-2011 Kirill Gavrilov <kirill@sview.ru>
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StGLTextureButton_h_
#define __StGLTextureButton_h_

#include <StGL/StGLVertexBuffer.h>
#include <StGLWidgets/StGLShare.h>
#include <StGLWidgets/StGLWidget.h>

class StGLTexture;
class ST_LOCAL StGLTextureButton : public StGLWidget {

        private:

    StGLVertexBuffer stVerticesBuf;
    StGLVertexBuffer stTexCoordBuf;

    size_t              faceId;
    size_t          facesCount;
    StGLTexture*    stTextures;
    StString* texturesPaths;

    class StButtonProgram;
    StGLShare<StButtonProgram> stProgram;

    StTimer        stWaveTimer;

    void glWaveTimerControl();

        public:

    StGLTextureButton(StGLWidget* parent,
                      const int& left = 32, const int& top = 32,
                      const StGLCorner corner = StGLCorner(ST_VCORNER_TOP, ST_HCORNER_LEFT),
                      const size_t& facesCount = 1);

    virtual ~StGLTextureButton();

    virtual const StString& getClassName();

    size_t getFaceId() {
        return faceId;
    }

    void setFaceId(const size_t& id) {
        faceId = id;
    }

    void setTexturePath(const StString* texturesPaths, const size_t& count = 1);

    virtual void stglResize(const StRectI_t& winRectPx);
    virtual bool stglInit();
    virtual void stglDraw(unsigned int view);

    virtual bool tryClick(const StPointD_t& cursorZo, const int& mouseBtn, bool& isItemClicked);
    virtual bool tryUnClick(const StPointD_t& cursorZo, const int& mouseBtn, bool& isItemUnclicked);

        private:   //!< callback Slots (private overriders)

    void doMouseUnclick(const int btnId);

        public:    //!< Signals

    struct {
        /**
         * Emit callback Slot on button click.
         * @param theUserData (const size_t ) - user predefined data.
         */
        StSignal<void (const size_t )> onBtnClick;
    } signals;

};

#endif //__StGLTextureButton_h_
