/**
 * Copyright © 2007-2012 Kirill Gavrilov
 *
 * Distributed under the Boost Software License, Version 1.0.
 * See accompanying file license-boost.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt
 */

#ifndef __StConfig_h_
#define __StConfig_h_

#include <StSettings/StConfigInterface.h>

class StLibrary;

class ST_LOCAL StConfig : public StConfigInterface {

        public:

    // typedef pointer-to-class
    typedef void* StConfig_t;

    // types definitions - needed for each exported function
    typedef StConfig_t (*StConfig_new_t)(const stUtf8_t* );
    typedef void (*StConfig_del_t)(StConfig_t );
    typedef stBool_t (*StConfig_loadInt32_t) (StConfig_t , const stUtf8_t* , int32_t& );
    typedef stBool_t (*StConfig_saveInt32_t) (StConfig_t , const stUtf8_t* , const int32_t& );
    typedef stBool_t (*StConfig_loadString_t)(StConfig_t , const stUtf8_t* , stUtf8_t* );
    typedef stBool_t (*StConfig_saveString_t)(StConfig_t , const stUtf8_t* , const stUtf8_t* );

    class ConfigFunctions {

            public:

        StConfig_new_t        StConfig_new;
        StConfig_del_t        StConfig_del;
        StConfig_loadInt32_t  StConfig_loadInt32;
        StConfig_saveInt32_t  StConfig_saveInt32;
        StConfig_loadString_t StConfig_loadString;
        StConfig_saveString_t StConfig_saveString;

            public:

        ConfigFunctions();

        void load(StLibrary& theLib);
        bool isNull() const;
        void nullify();

    };

    // exported functions' pointers
    static ConfigFunctions& GetFunctions();

        public:

    StConfig(const StString& theSettingsSet)
    : myLibInstance(NULL),
      myIsPointer(false) {
        myLibInstance = (StConfigInterface* )GetFunctions().StConfig_new(theSettingsSet.toCString());
    }

    StConfig(StConfigInterface* inst)
    : myLibInstance(inst),
      myIsPointer(true) {
        //
    }

    StConfigInterface* getLibImpl() {
        return myLibInstance;
    }

    virtual ~StConfig() {
        if(!myIsPointer) {
            GetFunctions().StConfig_del(myLibInstance);
        }
    }

    bool loadInt32(const StString& theParam,
                   int32_t&        theOutValue) {
        return GetFunctions().StConfig_loadInt32(myLibInstance, theParam.toCString(), theOutValue);
    }

    bool saveInt32(const StString& theParam,
                   const int32_t&  theValue) {
        return GetFunctions().StConfig_saveInt32(myLibInstance, theParam.toCString(), theValue);
    }

    bool loadString(const StString& theParam,
                    StString&       theOutValue) {
        stUtf8_t aBuffer[MAX_STRING_LENGHT];
        if(!GetFunctions().StConfig_loadString(myLibInstance, theParam.toCString(), aBuffer)) {
            return false;
        }
        theOutValue = StString(aBuffer);
        return true;
    }

    bool saveString(const StString& theParam,
                    const StString& theValue) {
        return GetFunctions().StConfig_saveString(myLibInstance, theParam.toCString(), theValue.toCString());
    }

        private:

    StConfigInterface* myLibInstance;
    bool               myIsPointer;

};

#endif //__StConfig_h_
